"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var handlebars_1 = require("handlebars");
var change_case_1 = require("change-case");
var common_tags_1 = require("common-tags");
var field_type_to_string_1 = require("./field-type-to-string");
var sanitizie_filename_1 = require("./sanitizie-filename");
var flatten_types_1 = require("./flatten-types");
exports.initHelpers = function (config, schemaContext) {
    handlebars_1.registerHelper('toPrimitive', function (type) {
        return config.primitives[type] || type || '';
    });
    handlebars_1.registerHelper('times', function (n, block) {
        var accum = '';
        for (var i = 0; i < n; ++i) {
            accum += block.fn(i);
        }
        return accum;
    });
    handlebars_1.registerHelper('ifDirective', function (context, directiveName, options) {
        if (context && context['directives'] && directiveName && typeof directiveName === 'string') {
            var directives = context['directives'];
            var directiveValue = directives[directiveName];
            if (directiveValue) {
                return options.fn ? options.fn(directiveValue) : '';
            }
        }
        return '';
    });
    handlebars_1.registerHelper('unlessDirective', function (context, directiveName, options) {
        if (context && context['directives'] && directiveName && typeof directiveName === 'string') {
            var directives = context['directives'];
            var directiveValue = directives[directiveName];
            if (!directiveValue) {
                return options.fn ? options.fn(directiveValue) : '';
            }
        }
        return '';
    });
    handlebars_1.registerHelper('toComment', function (str) {
        if (!str || str === '') {
            return '';
        }
        return '/* ' + (_a = ["", ""], _a.raw = ["", ""], common_tags_1.oneLineTrim(_a, str || '')) + ' */';
        var _a;
    });
    handlebars_1.registerHelper('eachImport', function (context, options) {
        var ret = '';
        var imports = [];
        // Interface, input types, types
        if (context.fields && !context.onType && !context.operationType) {
            context.fields.forEach(function (field) {
                if (!config.primitives[field.type]) {
                    var fieldType = field_type_to_string_1.getFieldTypeAsString(field);
                    var file = sanitizie_filename_1.sanitizeFilename(field.type, fieldType) + '.' + config.filesExtension;
                    if (!imports.find(function (t) { return t.name === field.type; })) {
                        imports.push({ name: field.type, file: file });
                    }
                    // Fields arguments
                    if (field.arguments && field.arguments.length > 0) {
                        field.arguments.forEach(function (arg) {
                            if (!config.primitives[arg.type]) {
                                var fieldType_1 = field_type_to_string_1.getFieldTypeAsString(arg);
                                var file_1 = sanitizie_filename_1.sanitizeFilename(arg.type, fieldType_1) + '.' + config.filesExtension;
                                if (!imports.find(function (t) { return t.name === arg.type; })) {
                                    imports.push({ name: arg.type, file: file_1 });
                                }
                            }
                        });
                    }
                }
            });
        }
        // Types that uses interfaces
        if (context.interfaces) {
            context.interfaces.forEach(function (infName) {
                var file = sanitizie_filename_1.sanitizeFilename(infName, 'interface') + '.' + config.filesExtension;
                if (!imports.find(function (t) { return t.name === infName; })) {
                    imports.push({ name: infName, file: file });
                }
            });
        }
        // Unions
        if (context.possibleTypes) {
            context.possibleTypes.forEach(function (possibleType) {
                var file = sanitizie_filename_1.sanitizeFilename(possibleType, 'type') + '.' + config.filesExtension;
                if (!imports.find(function (t) { return t.name === possibleType; })) {
                    imports.push({ name: possibleType, file: file });
                }
            });
        }
        // Operations and Fragments
        if (context.selectionSet) {
            var flattenDocument = context.isFlatten ? context : flatten_types_1.flattenSelectionSet(context);
            flattenDocument.innerModels.forEach(function (innerModel) {
                if (innerModel.fragmentsSpread && innerModel.fragmentsSpread.length > 0) {
                    innerModel.fragmentsSpread.forEach(function (fragmentSpread) {
                        var file = sanitizie_filename_1.sanitizeFilename(fragmentSpread.fragmentName, 'fragment') + '.' + config.filesExtension;
                        if (!imports.find(function (t) { return t.name === fragmentSpread.fragmentName; })) {
                            imports.push({ name: fragmentSpread.fragmentName, file: file });
                        }
                    });
                }
                var schemaType = innerModel.schemaBaseType;
                var inputType = schemaContext.inputTypes.find(function (inputType) { return inputType.name === schemaType; });
                if (inputType) {
                    var file = sanitizie_filename_1.sanitizeFilename(inputType.name, 'input-type') + '.' + config.filesExtension;
                    if (!imports.find(function (t) { return t.name === inputType.name; })) {
                        imports.push({ name: inputType.name, file: file });
                    }
                    return;
                }
                var enumType = schemaContext.enums.find(function (enumType) { return enumType.name === schemaType; });
                if (enumType) {
                    var file = sanitizie_filename_1.sanitizeFilename(enumType.name, 'enum') + '.' + config.filesExtension;
                    if (!imports.find(function (t) { return t.name === enumType.name; })) {
                        imports.push({ name: enumType.name, file: file });
                    }
                    return;
                }
                var scalarType = schemaContext.scalars.find(function (scalarType) { return scalarType.name === schemaType; });
                if (scalarType) {
                    var file = sanitizie_filename_1.sanitizeFilename(scalarType.name, 'scalar') + '.' + config.filesExtension;
                    if (!imports.find(function (t) { return t.name === scalarType.name; })) {
                        imports.push({ name: scalarType.name, file: file });
                    }
                    return;
                }
            });
        }
        for (var i = 0, j = imports.length; i < j; i++) {
            ret = ret + options.fn(imports[i]);
        }
        return ret;
    });
    handlebars_1.registerHelper('toLowerCase', function (str) {
        return (str || '').toLowerCase();
    });
    handlebars_1.registerHelper('toUpperCase', function (str) {
        return (str || '').toUpperCase();
    });
    handlebars_1.registerHelper('toPascalCase', function (str) {
        return change_case_1.pascalCase(str || '');
    });
    handlebars_1.registerHelper('toSnakeCase', function (str) {
        return change_case_1.snakeCase(str || '');
    });
    handlebars_1.registerHelper('toTitleCase', function (str) {
        return change_case_1.titleCase(str || '');
    });
    handlebars_1.registerHelper('toCamelCase', function (str) {
        return change_case_1.camelCase(str || '');
    });
    handlebars_1.registerHelper('multilineString', function (str) {
        if (!str) {
            return '';
        }
        var lines = str.split('\n');
        return lines.map(function (line, index) {
            var isLastLine = index != lines.length - 1;
            return "\"" + line.replace(/"/g, '\\"') + "\"" + (isLastLine ? ' +' : '');
        }).join('\r\n');
    });
    handlebars_1.registerHelper('for', function (from, to, incr, block) {
        var accum = '';
        for (var i = from; i < to; i += incr) {
            accum += block.fn(i);
        }
        return accum;
    });
};
//# sourceMappingURL=handlebars-extensions.js.map